@@ -1,9 +1,12 @@
+#![deny(clippy::dbg_macro)]
+
 use anyhow::{Context, Result};
 use clap::{Arg, ArgAction, ArgMatches, Command};
 use hgrep::grep::BufReadExt;
 use hgrep::printer::{PrinterOptions, TextWrapMode};
 use std::cmp;
 use std::env;
+use std::ffi::OsString;
 use std::io;
 use std::process;
 
@@ -20,6 +23,52 @@ use hgrep::bat::BatPrinter;
 use hgrep::syntect::SyntectPrinter;
 
 const COMPLETION_SHELLS: [&str; 6] = ["bash", "zsh", "powershell", "fish", "elvish", "nushell"];
+const OPTS_ENV_VAR: &str = "HGREP_DEFAULT_OPTS";
+
+#[derive(Debug)]
+struct Args {
+    env: Vec<String>,
+    args: env::ArgsOs,
+}
+
+impl Args {
+    fn new() -> Result<Self> {
+        let env = match env::var(OPTS_ENV_VAR) {
+            Ok(var) => {
+                let Some(mut opts) = shlex::split(&var) else {
+                    anyhow::bail!("String in `{}` environment variable cannot be parsed as a shell command: {:?}", OPTS_ENV_VAR, var);
+                };
+                opts.reverse();
+                opts
+            }
+            Err(env::VarError::NotPresent) => vec![],
+            Err(env::VarError::NotUnicode(invalid)) => {
+                anyhow::bail!(
+                    "String in `{}` environment variable is not a valid UTF-8 sequence: {:?}",
+                    OPTS_ENV_VAR,
+                    invalid,
+                );
+            }
+        };
+
+        let mut args = env::args_os();
+        args.next(); // Skip the executable name at the first item
+
+        Ok(Self { env, args })
+    }
+}
+
+impl Iterator for Args {
+    type Item = OsString;
+
+    fn next(&mut self) -> Option<Self::Item> {
+        if let Some(arg) = self.env.pop() {
+            Some(arg.into())
+        } else {
+            self.args.next()
+        }
+    }
+}
 
 fn command() -> Command {
     #[cfg(feature = "syntect-printer")]
@@ -31,11 +80,13 @@ fn command() -> Command {
     let cmd = Command::new("hgrep")
         .version(env!("CARGO_PKG_VERSION"))
         .about(
-            "hgrep is grep with human-friendly search output. It eats an output of `grep -nH` and prints the matches \
+            "hgrep is grep with human-friendly search output. hgrep eats an output of `grep -nH` and prints the matches \
             with syntax-highlighted code snippets.\n\n\
             $ grep -nH pattern -R . | hgrep\n\n\
-            For more details, visit https://github.com/rhysd/hgrep"
+            The default options can be customized with HGREP_DEFAULT_OPTS environment variable. \
+            For more details, visit https://github.com/rhysd/hgrep#readme"
         )
+        .no_binary_name(true)
         .arg(
             Arg::new("min-context")
                 .short('c')
@@ -168,13 +219,14 @@ fn command() -> Command {
     #[cfg(feature = "ripgrep")]
     let cmd = cmd
             .about(
-                "hgrep is grep with human-friendly search output. It eats an output of `grep -nH` and prints the \
-                matches with syntax-highlighted code snippets.\n\n\
+                "hgrep is grep with human-friendly search output.\n\n\
+                hgrep eats an output of `grep -nH` and prints the matches with syntax-highlighted code snippets.\n\n\
                 $ grep -nH pattern -R . | hgrep\n\n\
-                hgrep has its builtin subset of ripgrep. Its search output and performance are better than reading \
-                `grep -nH`.\n\n\
+                hgrep has its builtin subset of ripgrep, whose search output and performance are better than reading \
+                the output from `grep -nH`.\n\n\
                 $ hgrep pattern\n\n\
-                For more details, visit https://github.com/rhysd/hgrep"
+                The default options can be customized with HGREP_DEFAULT_OPTS environment variable. \
+                For more details, visit https://github.com/rhysd/hgrep#readme"
             )
             .override_usage("hgrep [FLAGS] [OPTIONS] [PATTERN [PATH...]]")
             .arg(
@@ -385,19 +437,19 @@ fn generate_completion_script<W: io::Write>(shell: &str, out: &mut W) {
 
     let mut cmd = command();
     if shell.eq_ignore_ascii_case("bash") {
-        generate(Bash, &mut cmd, "hgrep", out)
+        generate(Bash, &mut cmd, "hgrep", out);
     } else if shell.eq_ignore_ascii_case("zsh") {
-        generate(Zsh, &mut cmd, "hgrep", out)
+        generate(Zsh, &mut cmd, "hgrep", out);
     } else if shell.eq_ignore_ascii_case("powershell") {
-        generate(PowerShell, &mut cmd, "hgrep", out)
+        generate(PowerShell, &mut cmd, "hgrep", out);
     } else if shell.eq_ignore_ascii_case("fish") {
-        generate(Fish, &mut cmd, "hgrep", out)
+        generate(Fish, &mut cmd, "hgrep", out);
     } else if shell.eq_ignore_ascii_case("elvish") {
-        generate(Elvish, &mut cmd, "hgrep", out)
+        generate(Elvish, &mut cmd, "hgrep", out);
     } else if shell.eq_ignore_ascii_case("nushell") {
-        generate(Nushell, &mut cmd, "hgrep", out)
+        generate(Nushell, &mut cmd, "hgrep", out);
     } else {
-        unreachable!() // SHELL argument was validated by clap
+        unreachable!(); // SHELL argument was validated by clap
     }
 }
 
@@ -702,7 +754,7 @@ fn main() {
         process::exit(2);
     }
 
-    let status = match run(command().get_matches()) {
+    let status = match Args::new().and_then(|a| run(command().get_matches_from(a))) {
         Ok(true) => 0,
         Ok(false) => 1,
         Err(err) => {
@@ -721,17 +773,12 @@ fn main() {
 mod tests {
     use super::*;
 
+    const EMPTY: [OsString; 0] = [];
     #[cfg(not(windows))]
     const SNAPSHOT_DIR: &str = "../testdata/snapshots";
     #[cfg(windows)]
     const SNAPSHOT_DIR: &str = r#"..\testdata\snapshots"#;
 
-    fn cmdline<'a>(args: &[&'a str]) -> Vec<&'a str> {
-        let mut v = vec!["hgrep"];
-        v.extend(args);
-        v
-    }
-
     mod arg_matches {
         use super::*;
 
@@ -759,7 +806,7 @@ mod tests {
                     settings.set_snapshot_path(SNAPSHOT_DIR);
                     settings.bind(|| {
                         let cmd = command();
-                        let mat = cmd.get_matches_from(cmdline(&$args));
+                        let mat = cmd.get_matches_from($args);
                         let raw = get_raw_matched_arguments(&mat);
                         insta::assert_debug_snapshot!(raw);
                     });
@@ -767,7 +814,7 @@ mod tests {
             };
         }
 
-        snapshot_test!(no_arg, []);
+        snapshot_test!(no_arg, EMPTY);
         snapshot_test!(pat_only, ["pat"]);
         snapshot_test!(pat_and_dir, ["pat", "dir1"]);
         snapshot_test!(pat_and_dirs, ["pat", "dir1", "dir2", "dir3"]);
@@ -863,7 +910,7 @@ mod tests {
                 &["--printer", "bat", "--background"][..],
                 &["--printer", "bat", "--ascii-lines"][..],
             ] {
-                let mat = command().get_matches_from(cmdline(args));
+                let mat = command().get_matches_from(args);
                 assert!(run(mat).is_err(), "args: {:?}", args);
             }
         }
@@ -881,7 +928,7 @@ mod tests {
                 &["--wrap", "foo"][..],
                 &["--generate-completion-script", "unknown-shell"][..],
             ] {
-                let parsed = command().try_get_matches_from(cmdline(args));
+                let parsed = command().try_get_matches_from(args);
                 assert!(parsed.is_err(), "args: {:?}", args);
             }
         }
@@ -897,7 +944,7 @@ mod tests {
                     let mut settings = insta::Settings::clone_current();
                     settings.set_snapshot_path(SNAPSHOT_DIR);
                     settings.bind(|| {
-                        let mat = command().get_matches_from(cmdline(&$args));
+                        let mat = command().get_matches_from($args);
                         let min_ctx = mat
                             .get_one::<String>("min-context")
                             .unwrap()
@@ -916,7 +963,7 @@ mod tests {
             };
         }
 
-        snapshot_test!(no_arg, []);
+        snapshot_test!(no_arg, EMPTY);
         snapshot_test!(pat_only, ["pat"]);
         snapshot_test!(pat_and_dirs, ["pat", "dir1", "dir2"]);
         snapshot_test!(glob_one, ["--glob", "*.txt", "pat", "dir"]);
@@ -985,4 +1032,98 @@ mod tests {
             assert!(!v.is_empty(), "shell: {}", shell);
         }
     }
+
+    mod args {
+        use super::*;
+        use std::ffi::OsString;
+        use std::sync::Mutex;
+
+        struct Guard {
+            saved: Option<String>,
+        }
+        impl Guard {
+            fn new() -> Self {
+                Self {
+                    saved: env::var(OPTS_ENV_VAR).ok(),
+                }
+            }
+        }
+        impl Drop for Guard {
+            fn drop(&mut self) {
+                if let Some(v) = &self.saved {
+                    env::set_var(OPTS_ENV_VAR, v);
+                } else {
+                    env::remove_var(OPTS_ENV_VAR);
+                }
+            }
+        }
+
+        static MU: Mutex<()> = Mutex::new(());
+
+        #[test]
+        fn iterate_args() {
+            let _lock = MU.lock().unwrap();
+            let _guard = Guard::new();
+
+            for (env, prefix) in [
+                ("-i", &["-i"][..]),
+                ("-i -S", &["-i", "-S"][..]),
+                ("'-i'", &["-i"][..]),
+                ("'foo bar'", &["foo bar"][..]),
+                (r#""foo\\ bar""#, &[r#"foo\ bar"#][..]),
+                ("", &[][..]),
+            ] {
+                env::set_var(OPTS_ENV_VAR, env);
+
+                let have = Args::new().unwrap().collect::<Vec<_>>();
+                let mut want = prefix.iter().map(OsString::from).collect::<Vec<_>>();
+                let mut args = env::args_os();
+                args.next(); // Omit the executable name at the first argument
+                want.extend(args);
+
+                assert_eq!(want, have, "{env:?}, {prefix:?}");
+            }
+        }
+
+        #[test]
+        fn no_env_for_args() {
+            let _lock = MU.lock().unwrap();
+            let _guard = Guard::new();
+            env::remove_var(OPTS_ENV_VAR);
+
+            let have = Args::new().unwrap().collect::<Vec<_>>();
+            let mut want = env::args_os().collect::<Vec<_>>();
+            want.remove(0);
+            assert_eq!(want, have);
+        }
+
+        #[test]
+        fn broken_shell_command_in_env() {
+            let _lock = MU.lock().unwrap();
+            let _guard = Guard::new();
+            env::set_var(OPTS_ENV_VAR, "'-i");
+
+            let err = Args::new().unwrap_err();
+            let msg = format!("{}", err);
+            assert!(
+                msg.contains("cannot be parsed as a shell command"),
+                "{msg:?}",
+            );
+        }
+
+        #[test]
+        #[cfg(not(windows))]
+        fn invalid_utf8_sequence_in_env() {
+            use std::ffi::OsStr;
+            use std::os::unix::ffi::OsStrExt;
+
+            let _lock = MU.lock().unwrap();
+            let _guard = Guard::new();
+            env::set_var(OPTS_ENV_VAR, OsStr::from_bytes(b"\xc3\x28"));
+
+            let err = Args::new().unwrap_err();
+            let msg = format!("{}", err);
+            assert!(msg.contains("is not a valid UTF-8 sequence"), "{msg:?}");
+        }
+    }
 }
