@@ -1,3 +1,4 @@
+use crate::broken_pipe::IgnoreBrokenPipe as _;
 use crate::chunk::Files;
 use crate::grep::GrepMatch;
 use crate::printer::Printer;
@@ -383,7 +384,6 @@ impl<'main> Config<'main> {
             Ok(())
         }
 
-        use crate::broken_pipe::IgnoreBrokenPipe;
         let types = self.build_types()?;
         print(out, &types).ignore_broken_pipe()?;
         Ok(())
@@ -406,7 +406,7 @@ pub fn grep<'main, P: Printer + Sync>(
 
     let paths = entries.filter_map(|entry| match entry {
         Ok(entry) => {
-            if entry.file_type().map(|t| t.is_file()).unwrap_or(false) {
+            if entry.file_type().is_some_and(|t| t.is_file()) {
                 Some(Ok(entry.into_path()))
             } else {
                 None
